require 'disque'
require 'active_support/json'

module ActiveJob
  module QueueAdapters
    class DisqueAdapter
      def self.enqueue(job)
        client.push(
          job.queue_name,
          ActiveSupport::JSON.encode(job.serialize),
          @disque_timeout
        )
      end

      def self.enqueue_at(job, timestamp)
        client.push(
          job.queue_name,
          ActiveSupport::JSON.encode(job.serialize),
          @disque_timeout,
          delay: timestamp.to_i - Time.now.to_i
        )
      end

      def self.client(client: nil, timeout: nil)
        @disque_timeout ||= timeout || ENV.fetch('DISQUE_TIMEOUT', '100').to_i

        @disque_client ||= client || Disque.new(
          ENV.fetch('DISQUE_NODES', 'localhost:7711'),
          auth: ENV.fetch('DISQUE_AUTH', nil),
          cycle: ENV.fetch('DISQUE_CYCLE', '1000').to_i
        )
      end
    end
  end
end
